/* GStreamer
 * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
 *                    2000 Wim Taymans <wtay@chello.be>
 *
 * gstclock.c: Clock subsystem for maintaining time sync
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gstcomediclock.h"

static void 		gst_comedi_clock_class_init 	(GstComediClockClass *klass);
static void 		gst_comedi_clock_init 		(GstComediClock *clock);

static GstClockTime	gst_comedi_clock_get_internal_time 		(GstClock *clock);

static GstSystemClockClass *parent_class = NULL;
/* static guint gst_comedi_clock_signals[LAST_SIGNAL] = { 0 }; */
  
GType
gst_comedi_clock_get_type (void)
{ 
  static GType clock_type = 0;
	    
  if (!clock_type) {
    static const GTypeInfo clock_info = {
      sizeof (GstComediClockClass),
      NULL,
      NULL,
      (GClassInitFunc) gst_comedi_clock_class_init,
      NULL,
      NULL,
      sizeof (GstComediClock),
      4,
      (GInstanceInitFunc) gst_comedi_clock_init,
      NULL
    };
    clock_type = g_type_register_static (GST_TYPE_SYSTEM_CLOCK, "GstComediClock",
                                         &clock_info, 0);
  }
  return clock_type;
}


static void
gst_comedi_clock_class_init (GstComediClockClass *klass)
{
  GObjectClass *gobject_class;
  GstObjectClass *gstobject_class;
  GstClockClass *gstclock_class;

  gobject_class = (GObjectClass*) klass;
  gstobject_class = (GstObjectClass*) klass;
  gstclock_class = (GstClockClass*) klass;

  parent_class = g_type_class_ref (GST_TYPE_SYSTEM_CLOCK);

  gstclock_class->get_internal_time =	gst_comedi_clock_get_internal_time;
}

static void
gst_comedi_clock_init (GstComediClock *clock)
{
  gst_object_set_name (GST_OBJECT (clock), "GstComediClock");

  clock->prev1 = 0;
  clock->prev2 = 0;
}

GstComediClock*
gst_comedi_clock_new (gchar *name, GstComediClockGetTimeFunc func, gpointer user_data)
{
  GstComediClock *comedi_clock = GST_COMEDI_CLOCK (g_object_new (GST_TYPE_COMEDI_CLOCK, NULL));

  comedi_clock->func = func;
  comedi_clock->user_data = user_data;
  comedi_clock->adjust = 0;

  return comedi_clock;
}

void
gst_comedi_clock_set_active (GstClock *clock, gboolean active)
{
  GstComediClock *comedi_clock = GST_COMEDI_CLOCK (clock);
  GTimeVal timeval;
  GstClockTime time;
  GstClockTime comeditime;

  g_get_current_time (&timeval);
  time = GST_TIMEVAL_TO_TIME (timeval);
  comeditime = comedi_clock->func (clock, comedi_clock->user_data);

  if (active) {
    comedi_clock->adjust = time - comeditime;
  }
  else {
    comedi_clock->adjust = comeditime - time;
  }

  comedi_clock->active = active;
}

static GstClockTime
gst_comedi_clock_get_internal_time (GstClock *clock)
{
  GstComediClock *comedi_clock = GST_COMEDI_CLOCK (clock);
  
  if (comedi_clock->active) {
    GstClockTime comeditime;
    
    comeditime = comedi_clock->func (clock, comedi_clock->user_data) + comedi_clock->adjust;

    return comeditime;
  }
  else {
    GstClockTime time;
    GTimeVal timeval;
    
    g_get_current_time (&timeval);
    time = GST_TIMEVAL_TO_TIME (timeval);

    return time;
  }
}

