/* GStreamer
 * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
 *                    2000 Wim Taymans <wtay@chello.be>
 *                    2002 David A. Schleef <ds@schleef.org>
 *
 * gstcomedisrc.h: 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GST_COMEDISRC_H__
#define __GST_COMEDISRC_H__


#include <gst/gst.h>
#include <comedilib.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GST_TYPE_COMEDISRC \
  (gst_comedisrc_get_type())
#define GST_COMEDISRC(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_COMEDISRC,GstComediSrc))
#define GST_COMEDISRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_COMEDISRC,GstComediSrcClass))
#define GST_IS_COMEDISRC(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_COMEDISRC))
#define GST_IS_COMEDISRC_CLASS(obj) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_COMEDISRC))

typedef enum {
  GST_COMEDISRC_OPEN		= GST_ELEMENT_FLAG_LAST,

  GST_COMEDISRC_FLAG_LAST	= GST_ELEMENT_FLAG_LAST+2,
} GstComediSrcFlags;

typedef struct _GstComediSrc GstComediSrc;
typedef struct _GstComediSrcClass GstComediSrcClass;

struct _GstComediSrc {
  GstElement element;

  /* pads */
  GstPad *srcpad;

  /* device name */
  gchar *device;

  comedi_t *dev;
  gint fd;
  unsigned int subd;

  comedi_cmd cmd;
  unsigned int chanlist[16];

  char *read_buffer;
  guint read_buffer_len;

  /* audio parameters */
  gint format;
  gint channels;
  gint rate;
  guint interval;

  gboolean cmd_running;

  /* blocking */
  guint64 basetime;
  guint64 samples_since_basetime;
  gulong curoffset;
  gulong bytes_per_read;
};

struct _GstComediSrcClass {
  GstElementClass parent_class;
};

GType gst_comedisrc_get_type(void);

gboolean gst_comedisrc_factory_init (GstPlugin *plugin);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GST_COMEDISRC_H__ */
