
#include <glib.h>
#include <gst/gst.h>

#define INTERVAL0 5
#define INTERVAL1 7

int trial_index_0;
int trial_index_1;

int get_play_time_ms_0(void)
{
  static int time = 0;
  int interval;

  /* This creates a random number between 5000 and 10000 (milliseconds) */
  interval = g_random_int_range(5000,10000);
  time += interval;

  g_print("0 %d %d %d\n", trial_index_0, interval, time);

  return time;
}

int get_play_time_ms_1(void)
{
  static int time = 0;
  int interval;

  /* This creates a random number between 5000 and 10000 (milliseconds) */
  interval = g_random_int_range(5000,10000);
  time += interval;

  g_print("1 %d %d %d\n", trial_index_1, interval, time);

  return time;
}

char * get_file_name_0(void)
{
  return "out.wav";
}

char * get_file_name_1(void)
{
  return "out.wav";
}


/* GStreamer stuff */

static int debug = 0;

#define two
//#define fsink

GstElement *pipeline;

GstElement *src1;
GstElement *src2;

void set_play_time(GstElement *,int seconds);


void state_change(GstElement *element, gint arg1, gint arg2, gpointer closure)
{
  GstElementState state = gst_element_get_state(element);

  if(debug) g_print("state change %s to %s\n", GST_ELEMENT_NAME(element), gst_element_state_get_name(state));
}

void new_media1(GstElement *element, gpointer closure)
{
  if(debug) g_print("new_media1\n");
  set_play_time(gst_bin_get_by_name(GST_BIN(pipeline), "hold0"),
      get_play_time_ms_0());
  trial_index_0++;
}

void new_media2(GstElement *element, gpointer closure)
{
  if(debug) g_print("new_media2\n");

  set_play_time(gst_bin_get_by_name(GST_BIN(pipeline), "hold1"),
      get_play_time_ms_1());
  trial_index_1++;
}

void set_play_time(GstElement *element,int mseconds)
{
  GValue value = { 0 };

  g_value_init(&value, G_TYPE_UINT64);
  g_value_set_uint64(&value, mseconds*GST_MSECOND);
  gst_element_set_property(element, "play_time", &value);
}

GstElement *create_src(const char *name,const char *file)
{
  GstElement *src;
  GstElement *a,*b,*c;

  src = gst_pipeline_new(name);

  a = gst_element_factory_make("filesrc", NULL);
  g_assert(a);
  gst_util_set_object_arg(G_OBJECT(a), "location", file);
  b = gst_element_factory_make("wavparse", NULL);
  g_assert(b);
  c = gst_element_factory_make("queue", "queue");
  g_assert(c);

  gst_bin_add_many(GST_BIN(src),a,b,c,NULL);

  gst_element_link_many(a,b,c,NULL);

  g_signal_connect(G_OBJECT(src),
      "state_change", G_CALLBACK(state_change), NULL);

  return src;
}

GstElement *create_pipe(void)
{
  GstElement *pipe;
  GstElement *a,*b,*c,*d,*e;
  GstElement *t;

  pipe = gst_pipeline_new("pipe1");

  t = GST_ELEMENT(gst_thread_new("thread0"));
  g_assert(t);

  a = gst_element_factory_make("hold", "hold0");
  g_assert(a);
  g_signal_connect(G_OBJECT(a), "new_media", G_CALLBACK(new_media1), NULL);
  b = gst_element_factory_make("mono2stereo", NULL);
  g_assert(b);
  gst_util_set_object_arg(G_OBJECT(b), "pan","-1.0");
  c = gst_element_factory_make("queue", "thread_queue0");
  g_assert(c);

  gst_bin_add_many(GST_BIN(t),a,b,c,NULL);
  gst_element_link_many(a,b,c,NULL);

  d = gst_element_factory_make("adder", NULL);
  g_assert(d);

#ifdef fsink
  e = gst_element_factory_make("filesink", NULL);
  gst_util_set_object_arg(G_OBJECT(e), "location","out");
  g_assert(e);
#else
  e = gst_element_factory_make("esdsink", NULL);
  g_assert(e);
#endif

  gst_bin_add_many(GST_BIN(pipe),t,d,e,NULL);
  gst_element_link_many(c,d,e,NULL);

#ifdef two
  t = GST_ELEMENT(gst_thread_new("thread1"));
  g_assert(t);

  a = gst_element_factory_make("hold", "hold1");
  g_assert(a);
  g_signal_connect(G_OBJECT(a), "new_media", G_CALLBACK(new_media2), NULL);
  b = gst_element_factory_make("mono2stereo", NULL);
  g_assert(b);
  gst_util_set_object_arg(G_OBJECT(b), "pan","1.0");
  c = gst_element_factory_make("queue", "thread_queue1");
  g_assert(c);

  gst_bin_add_many(GST_BIN(t),a,b,c,NULL);
  gst_element_link_many(a,b,c,NULL);

  gst_bin_add_many(GST_BIN(pipe),t,NULL);
  gst_element_link_pads(c,"src",d,"sink%d");
#endif
 
  return pipe;
}

GstElement *create_pipe2(void)
{
  GstElement *pipe;
  GstElement *a,*b,*d,*e;

  pipe = gst_pipeline_new("pipe1");

  a = gst_element_factory_make("hold", "hold0");
  g_assert(a);
  g_signal_connect(G_OBJECT(a), "new_media", G_CALLBACK(new_media1), NULL);
  b = gst_element_factory_make("mono2stereo", NULL);
  g_assert(b);
  gst_util_set_object_arg(G_OBJECT(b), "pan","-1.0");

  gst_bin_add_many(GST_BIN(pipe),a,b,NULL);
  gst_element_link_many(a,b,NULL);

  d = gst_element_factory_make("adder", NULL);
  g_assert(d);

#ifdef fsink
  e = gst_element_factory_make("filesink", NULL);
  gst_util_set_object_arg(G_OBJECT(e), "location","out");
  g_assert(e);
#else
  e = gst_element_factory_make("esdsink", NULL);
  g_assert(e);
#endif

  gst_bin_add_many(GST_BIN(pipe),d,e,NULL);
  gst_element_link_many(b,d,e,NULL);

#ifdef two
  a = gst_element_factory_make("hold", "hold1");
  g_assert(a);
  g_signal_connect(G_OBJECT(a), "new_media", G_CALLBACK(new_media2), NULL);
  b = gst_element_factory_make("mono2stereo", NULL);
  g_assert(b);
  gst_util_set_object_arg(G_OBJECT(b), "pan","1.0");

  gst_bin_add_many(GST_BIN(pipe),a,b,NULL);
  gst_element_link_many(a,b,NULL);

  gst_element_link_pads(b,"src",d,"sink%d");
#endif
 
  return pipe;
}

int main (int argc, char *argv[])
{
  int ret;

  gst_init(&argc, &argv);
  gst_use_threads(TRUE);

  pipeline = create_pipe2();

  src1 = create_src("src1",get_file_name_0());
  gst_element_link(gst_bin_get_by_name(GST_BIN(src1), "queue"),
      gst_bin_get_by_name(GST_BIN(pipeline), "hold0"));

#ifdef two
  src2 = create_src("src2",get_file_name_1());
  gst_element_link(gst_bin_get_by_name(GST_BIN(src2), "queue"),
      gst_bin_get_by_name(GST_BIN(pipeline), "hold1"));
#endif

  trial_index_0=0;
  set_play_time(gst_bin_get_by_name(GST_BIN(pipeline), "hold0"),
      get_play_time_ms_0());
  trial_index_0++;

#ifdef two
  trial_index_1=0;
  set_play_time(gst_bin_get_by_name(GST_BIN(pipeline), "hold1"),
      get_play_time_ms_1());
  trial_index_1++;
#endif

  ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
  if(ret == GST_STATE_FAILURE){
    g_error("could not start pipeline");
  }
  ret = gst_element_set_state (src1, GST_STATE_PLAYING);
  if(ret == GST_STATE_FAILURE){
    g_error("could not start src1");
  }
#ifdef two
  ret = gst_element_set_state (src2, GST_STATE_PLAYING);
  if(ret == GST_STATE_FAILURE){
    g_error("could not start src2");
  }
#endif
  
  while (1){
    ret = gst_bin_iterate (GST_BIN (src1));
    if(!ret){
      gst_object_unref(GST_OBJECT(src1));
      src1 = create_src("src1",get_file_name_0());
      g_assert(src1);
      gst_element_link(gst_bin_get_by_name(GST_BIN(src1), "queue"),
          gst_bin_get_by_name(GST_BIN(pipeline), "hold0"));

      gst_element_set_state (src1, GST_STATE_PLAYING);
    }
#ifdef two
    ret = gst_bin_iterate (GST_BIN (src2));
    if(!ret){
      gst_object_unref(GST_OBJECT(src2));
      src2 = create_src("src2",get_file_name_1());
      g_assert(src2);
      gst_element_link(gst_bin_get_by_name(GST_BIN(src2), "queue"),
          gst_bin_get_by_name(GST_BIN(pipeline), "hold1"));

      gst_element_set_state (src2, GST_STATE_PLAYING);
    }
#endif
    ret = gst_bin_iterate (GST_BIN (pipeline));
  }

  gst_element_set_state (pipeline, GST_STATE_NULL);
  gst_element_set_state (src1, GST_STATE_NULL);

  return 0;
}

